<?php

namespace Drupal\vegas\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media\Entity\Media;

/**
 * VegasConfigForm Class.
 *
 * Configuration form for vegas configuration entity.
 */
class VegasConfigForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function gettransitionarray() {
    $transition_array = [
      'fade' => 'fade (default)',
      'fade2' => 'fade2',
      'slideLeft' => 'slideLeft',
      'slideLeft2' => 'slideLeft2',
      'slideRight' => 'slideRight',
      'slideRight2' => 'slideRight2',
      'slideUp' => 'slideUp',
      'slideUp2' => 'slideUp2',
      'slideDown' => 'slideDown',
      'slideDown2' => 'slideDown2',
      'zoomIn' => 'zoomIn',
      'zoomIn2' => 'zoomIn2',
      'zoomOut' => 'zoomOut',
      'zoomOut2' => 'zoomOut2',
      'swirlLeft' => 'swirlLeft',
      'swirlLeft2' => 'swirlLeft2',
      'swirlRight' => 'swirlRight',
      'swirlRight2' => 'swirlRight2',
      'burn' => 'burn',
      'burn2' => 'burn2',
      'blur' => 'blur',
      'blur2' => 'blur2',
      'flash' => 'flash',
      'flash2' => 'flash2',
    ];
    return $transition_array;
  }

  /**
   * {@inheritdoc}
   */
  public function getanimationarray() {
    $animation_array = [
      'kenburns' => 'kenburns',
      'kenburnsUp' => 'kenburnsUp',
      'kenburnsDown' => 'kenburnsDown',
      'kenburnsRight' => 'kenburnsRight',
      'kenburnsLeft' => 'kenburnsLeft',
      'kenburnsUpLeft' => 'kenburnsUpLeft',
      'kenburnsUpRight' => 'kenburnsUpRight',
      'kenburnsDownLeft' => 'kenburnsDownLeft',
      'kenburnsDownRight' => 'kenburnsDownRight',
    ];
    return $animation_array;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $vegas_config = $this->entity;

    /* Appearance */
    $current_config_vegas_timer = $vegas_config->get('config_vegas_timer');
    $current_config_vegas_overlay = $vegas_config->get('config_vegas_overlay');
    $current_config_vegas_delay = $vegas_config->get('config_vegas_delay');
    $current_config_vegas_color = $vegas_config->get('config_vegas_color');
    $current_config_vegas_cover = $vegas_config->get('config_vegas_cover');

    /* ToDo: Behavior */

    /* Transition */
    $current_config_vegas_transition_select = $vegas_config->get('config_vegas_transition_select');
    $current_config_vegas_transition = $vegas_config->get('config_vegas_transition');
    $current_config_vegas_transition_duration_select = $vegas_config->get('config_vegas_transition_duration_select');
    $current_config_vegas_transition_duration = $vegas_config->get('config_vegas_transition_duration');

    /* Animation */
    $current_config_vegas_animation_select = $vegas_config->get('config_vegas_animation_select');
    $current_config_vegas_animation = $vegas_config->get('config_vegas_animation');
    $current_config_vegas_animation_duration_select = $vegas_config->get('config_vegas_animation_duration_select');
    $current_config_vegas_animation_duration = $vegas_config->get('config_vegas_animation_duration');

    /* Path */
    $current_config_vegas_path = $vegas_config->get('config_vegas_path');

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $vegas_config->label(),
      '#description' => $this->t("Label for the Vegas."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $vegas_config->id(),
      '#machine_name' => [
        'exists' => '\Drupal\vegas\Entity\VegasConfig::load',
      ],
      '#disabled' => !$vegas_config->isNew(),
    ];

    $form['vegasform'] = [
      '#type' => 'vertical_tabs',
    ];

    $form['vegas_slides_details'] = [
      '#type' => 'details',
      '#title' => t('Vegas slides'),
      '#open' => TRUE,
      '#group' => 'vegasform',
    ];

    $form['vegas_slides_details']['vegas_slides_table'] = [
      '#type' => 'table',
      '#tree' => TRUE,
      '#prefix' => '<div id="vegas-wrapper">',
      '#suffix' => '</div>',
      '#header' => [
        t('Item'),
        t('Weight'),
        t('Slide'),
        /* ToDo: t('Custom slide settings'), */
        t('Actions'),
      ],
      '#tabledrag' => [
              [
                'action' => 'order',
                'relationship' => 'sibling',
                'group' => 'mytable-order-weight',
              ],
      ],
    ];

    $weights = $form_state->getValue('vegas_slides_table');

    if (empty($weights)) {
      $weights = $vegas_config->get('vegas_slides_table');
    }

    if (empty($weights)) {
      $weights[] = [
        'weight' => 0,
        'vegas_single_slide' => [],
      ];
    }

    uasort($weights, 'Drupal\Component\Utility\SortArray::sortByWeightElement');

    /* Vegas Slide */
    foreach ($weights as $delta => $weight) {
      $form['vegas_slides_details']['vegas_slides_table'][$delta]['#attributes']['class'][] = 'draggable';
      $form['vegas_slides_details']['vegas_slides_table'][$delta]['#weight'] = isset($weight['weight']) ? $weight['weight'] : 0;
      $form['vegas_slides_details']['vegas_slides_table'][$delta]['item'] = [
        '#plain_text' => $weight['weight'] ?? 0,
      ];

      /* Weight */
      $form['vegas_slides_details']['vegas_slides_table'][$delta]['weight'] = [
        '#type' => 'weight',
        '#title' => t('Weight for @title', ['@title' => 'vegas_single_slide']),
        '#title_display' => 'invisible',
        '#attributes' => ['class' => ['mytable-order-weight']],
        '#default_value' => $weight['weight'] ?? 0,
      ];

      /* Slides */
      $form['vegas_slides_details']['vegas_slides_table'][$delta]['vegas_single_slide']['slide'] = [
        '#type' => 'entity_autocomplete',
        '#title' => $this->t('Vegas Slide'),
        '#target_type' => 'media',
        '#default_value' => isset($weight['vegas_single_slide']['slide'][0]['target_id']) ? Media::load($weight['vegas_single_slide']['slide'][0]['target_id']) : '',
        '#handler' => 'default:media',
        '#handler_settings' => [
          'target_bundles' => [
                    [
                      'image',
                      'video',
                    ],
          ],
          'sort' => [
            'field' => '_none',
          ],
          'auto_create' => FALSE,
          'auto_create_bundle' => '',
        ],
        '#tags' => TRUE,
        '#size' => 40,
        '#maxlength' => 100,
      ];

      /* ToDo: Custom slide settings  */
      /* ToDo: Slide settings: Horizontal alignment */
      /* ToDo: Slide settings: Vertical alignment */
      /* ToDo: Slide settings: Transitions */

      if (count($weights) > 1) {
        $form['vegas_slides_details']['vegas_slides_table'][$delta]['delete'] = [
          '#type' => 'submit',
          '#title' => t('Remove'),
          '#name' => 'delete_' . $delta,
          '#value' => 'Remove',
          '#submit' => ['::ajaxSubmit'],
          '#ajax' => [
            'callback' => '::addMoreSet',
            'wrapper' => 'vegas-wrapper',
          ],
        ];
      }
      else {
        $form['vegas_slides_details']['vegas_slides_table'][$delta]['delete'] = [];
      }
    }

    $form['vegas_slides_details']['add'] = [
      '#type' => 'submit',
      '#title' => t('Add slide'),
      '#value' => t('Add one more slide'),
      '#submit' => ['::ajaxSubmit'],
      '#ajax' => [
        'callback' => '::addMoreSet',
        'wrapper' => 'vegas-wrapper',
      ],
    ];

    /* ToDo: Initial */

    /* Appearance */
    $form['vegas_appearance_details'] = [
      '#type' => 'details',
      '#title' => t('Appearance'),
      '#description' => t('Vegas settings for the presentation.'),
      '#open' => TRUE,
      '#group' => 'vegasform',
    ];

    $form['vegas_appearance_details']['config_vegas_timer'] = [
      '#type' => 'select',
      '#title' => $this->t('Timer bar'),
      '#options' => [
        0 => "False",
        1 => "True",
      ],
      '#default_value' => $current_config_vegas_timer ?? '1',
      '#description' => $this->t('Display or hide timer bar.'),
    ];

    $form['vegas_appearance_details']['config_vegas_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Color'),
      '#default_value' => $current_config_vegas_color ?? '#000000',
      '#description' => $this->t('Slide background color.'),
    ];

    $form['vegas_appearance_details']['config_vegas_cover'] = [
      '#type' => 'select',
      '#title' => $this->t('Cover'),
      '#options' => [
        'true' => "the slide image is scaled to fit the container",
        'false' => "the slide image is displayed entirely",
        'repeat' => "the slide image is repeated",
      ],
      '#default_value' => $current_config_vegas_cover ?? '1',
      '#description' => $this->t('How the slide Is displayed.'),
    ];

    $config_overlay_array = [
      '0' => 'false',
      '/libraries/vegas/overlays/01.png' => 'Vegas Included overlay 1',
      '/libraries/vegas/overlays/02.png' => 'Vegas Included overlay 2',
      '/libraries/vegas/overlays/03.png' => 'Vegas Included overlay 3',
      '/libraries/vegas/overlays/04.png' => 'Vegas Included overlay 4',
      '/libraries/vegas/overlays/05.png' => 'Vegas Included overlay 5',
      '/libraries/vegas/overlays/06.png' => 'Vegas Included overlay 6',
      '/libraries/vegas/overlays/07.png' => 'Vegas Included overlay 7',
      '/libraries/vegas/overlays/08.png' => 'Vegas Included overlay 8',
      '/libraries/vegas/overlays/09.png' => 'Vegas Included overlay 9',
    ];

    $form['vegas_appearance_details']['config_vegas_overlay'] = [
      '#type' => 'select',
      '#title' => $this->t('Overlay'),
      '#options' => $config_overlay_array,
      '#default_value' => $current_config_vegas_overlay ?? '0',
      '#description' => $this->t('Display, hide ore select the overlay. Overlays are repeated dot pattern that can be applied to reduce artefact effects on scaled backgrounds.'),
    ];

    /* Behavior */
    $form['vegas_behavior_details'] = [
      '#type' => 'details',
      '#title' => t('Behaviour'),
      '#open' => TRUE,
      '#group' => 'vegasform',
    ];

    $form['vegas_behavior_details']['config_vegas_delay'] = [
      '#type' => 'number',
      '#title' => 'Delay',
      '#step' => 1,
      '#min' => 1,
      '#max' => 100000,
      '#default_value' => $current_config_vegas_delay ?? '5000',
      '#description' => $this->t('Delay beetween slides in milliseconds..'),
    ];

    /* Global Transitions */
    $form['vegas_transition_details'] = [
      '#type' => 'details',
      '#title' => t('Transitions'),
      '#description' => t('Set the transition between slides.'),
      '#open' => TRUE,
      '#group' => 'vegasform',
    ];

    $form['vegas_transition_details']['config_vegas_transition_select'] = [
      '#type' => 'select',
      '#title' => $this->t('Slide Transition(s)'),
      '#options' => [
        'random' => "random",
        'custom' => "custom",
      ],
      '#default_value' => $current_config_vegas_transition_select ?? 'custom',
      '#description' => $this->t('Slide transition(s) between slides. If random is selected, one transition is automatically selected from all.'),
    ];

    $config_transition_array = $this->gettransitionarray();

    // Transition fade
    // Set the transition between slides.
    // Could be a transition name, random or an array of transition picked randomly.
    // Read the Transition section for more information.
    $form['vegas_transition_details']['config_vegas_transition'] = [
      '#type' => 'select',
      '#title' => $this->t('Transition'),
      '#options' => $config_transition_array,
      '#multiple' => 'TRUE',
      '#size' => 10,
      '#default_value' => $current_config_vegas_transition ?? 'fade',
      '#description' => $this->t('Select one or more transitions. If several are selected, one transition is chosen at random.'),
      '#states' => [
        'visible' => [
          'select[name="config_vegas_transition_select"]' => [
            'value' => 'custom',
          ],
        ],
      ],
    ];

    $form['vegas_transition_details']['config_vegas_transition_duration_select'] = [
      '#type' => 'select',
      '#title' => $this->t('Slide transition'),
      '#options' => [
        'random' => "auto",
        'custom' => "custom",
      ],
      '#default_value' => $current_config_vegas_transition_duration_select ?? 'custom',
      '#description' => $this->t('Set to auto so the transition duration will be equal to the slide delay.'),
    ];

    // Transition Duration 1000
    // Set the transition duration in milliseconds.
    // Could be auto so the transition duration will be equal to the slide delay .
    // config_vegas_transition_duration:
    $form['vegas_transition_details']['config_vegas_transition_duration'] = [
      '#type' => 'number',
      '#title' => 'Custom transition duration',
      '#step' => 1,
      '#min' => 1,
      '#max' => 100000,
      '#default_value' => $current_config_vegas_transition_duration ?? '2000',
      '#description' => $this->t('Set transition duration in milliseconds. (default: 1000)'),
      '#states' => [
        'visible' => [
          'select[name="config_vegas_transition_duration_select"]' => [
            'value' => 'custom',
          ],
        ],
      ],
    ];

    /* Global Animation */
    $form['vegas_animation_details'] = [
      '#type' => 'details',
      '#title' => t('Animation'),
      '#description' => t('Set the animation of the slides.'),
      '#open' => TRUE,
      '#group' => 'vegasform',
    ];

    // Set the animation of the slides. Could be an animation name, random or an array of transition picked randomly.
    // Read the Transition section for more information.
    // config_vegas_animation:
    $form['vegas_animation_details']['config_vegas_animation_select'] = [
      '#type' => 'select',
      '#title' => $this->t('Animation'),
      '#options' => [
        '' => "Disabled",
        'random' => "Random",
        'custom' => "Custom",
      ],
      '#default_value' => $current_config_vegas_animation_select ?? 'false',
      '#description' => $this->t('Set the animation of the slides. Could be an animation name, random or an array of transition picked randomly.'),
    ];

    $config_animation_array = $this->getanimationarray();

    $form['vegas_animation_details']['config_vegas_animation'] = [
      '#type' => 'select',
      '#title' => $this->t('Animation'),
      '#options' => $config_animation_array,
      '#multiple' => 'TRUE',
      '#size' => 10,
      '#default_value' => $current_config_vegas_animation ?? 'kenburns',
      '#description' => $this->t('Select one or more animations. If several are selected, one animation is chosen at random.'),
      '#states' => [
        'visible' => [
          'select[name="config_vegas_animation_select"]' => [
            'value' => 'custom',
          ],
        ],
      ],
    ];

    // @todo Form State => Auto or custom modification
    // animation Duration auto
    // Set the animation duration in milliseconds.
    // Could be auto so the animation duration will be equal to the slide delay .
    // config_vegas_animation_duration:
    $config_animation_duration_array = [
      'auto' => 'auto',
      'custom' => 'custom',
    ];

    $form['vegas_animation_details']['config_vegas_animation_duration_select'] = [
      '#type' => 'select',
      '#title' => $this->t('Animation duration'),
      '#states' => [
        'invisible' => [
          'select[name="config_vegas_animation"]' => [
            'value' => 'disabled',
          ],
        ],
      ],
      '#options' => $config_animation_duration_array,
      '#default_value' => $current_config_vegas_animation_duration_select ?? 'auto',
      '#description' => $this->t('Set the animation duration in milliseconds. Could be auto so the animation duration will be equal to the slide delay.'),
    ];

    $form['vegas_animation_details']['config_vegas_animation_duration'] = [
      '#type' => 'number',
      '#title' => 'Animation duration',
      '#step' => 1,
      '#min' => 0,
      '#max' => 100000,
      '#states' => [
        'visible' => [
          'select[name="config_vegas_animation_duration_select"]' => [
            'value' => 'custom',
          ],
        ],
        'invisible' => [
          'select[name="config_vegas_animation_duration_select"]' => [
            'value' => 'auto',
          ],
        ],
      ],
      '#default_value' => $current_config_vegas_animation_duration ?? '1000',
      '#description' => $this->t('Set the animation duration in milliseconds. <br /><b>Set to 0 (zero) so the transition duration will be equal to the slide delay. (default)</b>'),
    ];

    /* Path */
    $form['vegas_paths_details'] = [
      '#type' => 'details',
      '#title' => t('Paths'),
      '#description' => t('Vegas path settings.'),
      '#open' => TRUE,
      '#group' => 'vegasform',
    ];

    $form['vegas_paths_details']['config_vegas_path'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Paths'),
      '#default_value' => $current_config_vegas_path,
      '#description' => $this->t('New line separated paths that must start with a leading slash. Wildcard character is *. E.g. /comment/*/reply.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function addMoreSet(array &$form, FormStateInterface $form_state) {
    return $form['vegas_slides_details']['vegas_slides_table'];
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxSubmit(array &$form, FormStateInterface $form_state) {
    $weights = $form_state->getValue('vegas_slides_table');
    $parents = $form_state->getTriggeringElement()['#parents'];

    if (isset($parents[0]) && $parents[0] == 'add') {
      $weights[] = [
        'weight' => 0,
        'vegas_single_slide' => [],
      ];
    }

    if (isset($parents[2]) && $parents[2] == 'delete') {
      unset($weights[$parents[1]]);
    }

    $form_state->setValue('vegas_slides_table', $weights);
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {

    $vegas_config = $this->entity;
    $weights = $form_state->getValue('vegas_slides_table');
    $vegas_config->set('vegas_slides_table', array_values(array_filter($weights)));
    $status = $vegas_config->save();

    if ($status === SAVED_NEW) {
      $this->messenger()->addMessage($this->t('%label vegas configuration was created.', [
        '%label' => $vegas_config->label(),
      ]));
    }
    else {
      $this->messenger()->addMessage($this->t('%label vegas configuration has been updated.', [
        '%label' => $vegas_config->label(),
      ]));
    }
    $form_state->setRedirectUrl($vegas_config->toUrl('collection'));
  }

}
